@@ -19,7 +19,7 @@ from rest_framework.schemas.utils import get_pk_description, is_list_view
 
 from drf_spectacular.app_settings import spectacular_settings
 from drf_spectacular.types import OpenApiTypes, resolve_basic_type, PYTHON_TYPE_MAPPING, OPENAPI_TYPE_MAPPING
-from drf_spectacular.utils import PolymorphicResponse
+from drf_spectacular.utils import PolymorphicProxySerializer
 
 AUTHENTICATION_SCHEMES = {
     cls.authentication_class: cls for cls in spectacular_settings.SCHEMA_AUTHENTICATION_CLASSES
@@ -35,12 +35,19 @@ def anyisinstance(obj, type_list):
 
 
 def force_serializer_instance(serializer):
-    if inspect.isclass(serializer) and issubclass(serializer, serializers.Serializer):
+    if inspect.isclass(serializer) and issubclass(serializer, serializers.BaseSerializer):
         return serializer()
     else:
         return serializer
 
 
+def is_serializer(obj):
+    return anyisinstance(
+        force_serializer_instance(obj),
+        [serializers.BaseSerializer, PolymorphicProxySerializer]
+    )
+
+
 class ComponentRegistry:
     def __init__(self):
         self.schemas = {}
@@ -348,15 +355,23 @@ class AutoSchema(ViewInspector):
             return resolve_basic_type(OpenApiTypes.STR)
 
     def _map_serializer_field(self, method, field):
+        if hasattr(field, '_spectacular_annotation'):
+            if field._spectacular_annotation in OPENAPI_TYPE_MAPPING or field._spectacular_annotation in PYTHON_TYPE_MAPPING:
+                return resolve_basic_type(field._spectacular_annotation)
+            else:
+                return self._map_serializer_field(method, field._spectacular_annotation)
+
+        # nested serializer
+        if isinstance(field, serializers.Serializer):
+            return self.resolve_serializer(method, field, nested=True)
+
+        # nested serializer with many=True gets automatically replaced with ListSerializer
         if isinstance(field, serializers.ListSerializer):
             return {
                 'type': 'array',
                 'items': self.resolve_serializer(method, field.child)
             }
 
-        if isinstance(field, serializers.Serializer):
-            return self.resolve_serializer(method, field, nested=True)
-
         # Related fields.
         if isinstance(field, serializers.ManyRelatedField):
             return {
@@ -419,9 +434,11 @@ class AutoSchema(ViewInspector):
             return resolve_basic_type(OpenApiTypes.UUID)
 
         if isinstance(field, serializers.IPAddressField):
-            if 'IPv4' == field.protocol:
+            # TODO this might be a DRF bug. protocol is not propagated to serializer although it
+            #  should have been. results in always 'both' (thus no format)
+            if 'ipv4' == field.protocol.lower():
                 return resolve_basic_type(OpenApiTypes.IP4)
-            elif 'IPv6' == field.protocol:
+            elif 'ipv6' == field.protocol.lower():
                 return resolve_basic_type(OpenApiTypes.IP6)
             else:
                 return resolve_basic_type(OpenApiTypes.STR)
@@ -453,7 +470,8 @@ class AutoSchema(ViewInspector):
             return content
 
         if isinstance(field, serializers.FileField):
-            return resolve_basic_type(OpenApiTypes.BINARY)
+            # TODO returns filename. but does it accept binary data on upload?
+            return resolve_basic_type(OpenApiTypes.STR)
 
         if isinstance(field, serializers.SerializerMethodField):
             method = getattr(field.parent, field.method_name)
@@ -483,6 +501,39 @@ class AutoSchema(ViewInspector):
             content['minimum'] = field.min_value
 
     def _map_serializer(self, method, serializer, nested=False):
+        if isinstance(serializer, PolymorphicProxySerializer):
+            return self._map_polymorphic_proxy_serializer(method, serializer, nested=False)
+        else:
+            return self._map_concrete_serializer(method, serializer, nested=False)
+
+    def _map_polymorphic_proxy_serializer(self, method, serializer, nested):
+        """ custom handling for @extend_schema's injection of PolymorphicProxySerializer """
+        poly_list = []
+
+        for sub_serializer in serializer.serializers:
+            assert is_serializer(sub_serializer), 'sub-serializer must be either a Serializer or a PolymorphicProxySerializer.'
+            sub_serializer = force_serializer_instance(sub_serializer)
+
+            if serializer.resource_type_field_name not in sub_serializer.fields:
+                warn(
+                    f'sub-serializer of {serializer.component_name} must have the specified '
+                    f'discriminator field "{serializer.resource_type_field_name}".'
+                )
+
+            sub_schema = self.resolve_serializer(method, sub_serializer, nested)
+            sub_serializer_name = self._get_serializer_name(method, sub_serializer, nested)
+            poly_list.append((sub_serializer_name, sub_schema))
+
+        return {
+            'oneOf': [ref for _, ref in poly_list],
+            'discriminator': {
+                'propertyName': serializer.resource_type_field_name,
+                # TODO mapping name is not sourced from serializer. API breaks schema if it does not use component name
+                'mapping': {name: ref['$ref'] for name, ref in poly_list}
+            }
+        }
+
+    def _map_concrete_serializer(self, method, serializer, nested):
         required = []
         properties = {}
 
@@ -520,7 +571,6 @@ class AutoSchema(ViewInspector):
 
     def _map_field_validators(self, field, schema):
         for v in field.validators:
-            # "Formats such as "email", "uuid", and so on, MAY be used even though undefined by this specification."
             # https://github.com/OAI/OpenAPI-Specification/blob/master/versions/3.0.3.md#data-types
             if isinstance(v, validators.EmailValidator):
                 schema['format'] = 'email'
@@ -552,22 +602,15 @@ class AutoSchema(ViewInspector):
                     schema['maximum'] = int(digits * '9') + 1
                     schema['minimum'] = -schema['maximum']
 
-    def _map_type_hint(self, method, hint=None):
-        if not hint:
-            hint = typing.get_type_hints(method).get('return')
+    def _map_type_hint(self, method):
+        hint = getattr(method, '_spectacular_annotation', None) or typing.get_type_hints(method).get('return')
 
-        if hasattr(method, '_spectacular_annotation'):
-            if isinstance(force_serializer_instance(method._spectacular_annotation), serializers.BaseSerializer):
-                return self._map_serializer_field(method, force_serializer_instance(method._spectacular_annotation))
-            elif method._spectacular_annotation in OPENAPI_TYPE_MAPPING:
-                return resolve_basic_type(method._spectacular_annotation)
-            else:
-                warn(f'unknown decorated type "{repr(method._spectacular_annotation)}". defaulting to string')
-                return resolve_basic_type(OpenApiTypes.STR)
-        elif hint in PYTHON_TYPE_MAPPING:
+        if is_serializer(hint):
+            return self._map_serializer_field(method, force_serializer_instance(hint))
+        elif hint in PYTHON_TYPE_MAPPING or hint in OPENAPI_TYPE_MAPPING:
             return resolve_basic_type(hint)
         else:
-            warn(f'type hint for SerializerMethodField function "{method.__name__}" is unknown. defaulting to string.')
+            warn(f'type hint for function "{method.__name__}" is unknown. defaulting to string.')
             return resolve_basic_type(OpenApiTypes.STR)
 
     def _get_paginator(self):
@@ -605,12 +648,13 @@ class AutoSchema(ViewInspector):
             return None
 
     def _get_request_body(self, path, method):
+        # only unsafe methods can have a body
         if method not in ('PUT', 'PATCH', 'POST'):
             return {}
 
         serializer = force_serializer_instance(self.get_request_serializer(path, method))
 
-        if isinstance(serializer, serializers.Serializer):
+        if is_serializer(serializer):
             schema = self.resolve_serializer(method, serializer)
         else:
             warn(
@@ -636,7 +680,7 @@ class AutoSchema(ViewInspector):
     def _get_response_bodies(self, path, method):
         response_serializers = self.get_response_serializers(path, method)
 
-        if anyisinstance(force_serializer_instance(response_serializers), [serializers.Serializer, PolymorphicResponse]):
+        if is_serializer(response_serializers):
             if method == 'DELETE':
                 return {'204': {'description': 'No response body'}}
             return {'200': self._get_response_for_code(path, method, response_serializers)}
@@ -660,26 +704,10 @@ class AutoSchema(ViewInspector):
 
         if not serializer:
             return {'description': 'No response body'}
-        elif isinstance(serializer, serializers.Serializer):
+        elif anyisinstance(serializer, [serializers.Serializer, PolymorphicProxySerializer]):
             schema = self.resolve_serializer(method, serializer)
             if not schema:
                 return {'description': 'No response body'}
-        elif isinstance(serializer, PolymorphicResponse):
-            # custom handling for @extend_schema's injection of polymorphic responses
-            schemas = []
-
-            for serializer in serializer.serializers:
-                assert isinstance(serializer, serializers.Serializer)
-                schema_option = self.resolve_serializer(method, serializer)
-                if schema_option:
-                    schemas.append(schema_option)
-
-            schema = {
-                'oneOf': schemas,
-                'discriminator': {
-                    'propertyName': serializer.resource_type_field_name
-                }
-            }
         elif isinstance(serializer, serializers.ListSerializer):
             schema = self.resolve_serializer(method, serializer.child)
         elif isinstance(serializer, dict):
@@ -709,6 +737,9 @@ class AutoSchema(ViewInspector):
         }
 
     def _get_serializer_name(self, method, serializer, nested):
+        if isinstance(serializer, PolymorphicProxySerializer):
+            return serializer.component_name
+
         name = serializer.__class__.__name__
 
         if name.endswith('Serializer'):
@@ -737,9 +768,11 @@ class AutoSchema(ViewInspector):
             self.registry.schemas[name] = None
 
             schema = self._map_serializer(method, serializer, nested)
-            # empty serializer - usually a transactional serializer.
-            # no need to put it explicitly in the spec
-            if not schema['properties']:
+            # 3 cases:
+            #   1. polymorphic container component -> use
+            #   2. concrete component with properties -> use
+            #   3. concrete component without properties -> prob. transactional so discard
+            if 'oneOf' not in schema and not schema['properties']:
                 del self.registry.schemas[name]
                 return {}
             else:
