@@ -5,8 +5,18 @@ from rest_framework.settings import api_settings
 from drf_spectacular.types import resolve_basic_type
 
 
-class PolymorphicResponse:
-    def __init__(self, serializers, resource_type_field_name):
+class PolymorphicProxySerializer:
+    """
+    This class is to be used with @extend_schema to signal a request/response
+    might be polymorphic (accepts/returns data possibly from different serializers)
+
+    Beware that this is not a real serializer and therefore is not derived
+    from serializers.Serializer. It *cannot* be used in views as serializer_class
+    or as field in a actual serializer. You likely want to handle this in the view
+    method.
+    """
+    def __init__(self, component_name, serializers, resource_type_field_name):
+        self.component_name = component_name
         self.serializers = serializers
         self.resource_type_field_name = resource_type_field_name
 
@@ -127,7 +137,13 @@ def extend_schema(
 
 def extend_schema_field(response):
     """
-    decorator for SerializerMethodField(). always takes precedence over a type hint (if one was given).
+    Decorator for the "field" kind. Can be used with ``SerializerMethodField()`` (annotate the actual method)
+    or with custom ``serializers.Field`` implementations.
+
+    If your custom serializer field base class is already the desired type, decoration is not necessary. To
+    override the discovered base class type, you can decorate your custom field class.
+
+    Always takes precedence over other mechanisms (e.g. type hints, auto-discovery).
 
     :param response: accepts a Serializer or OpenApiTypes
     :return:
